# Program:	motion2.sh
# 		Motion V2 Database report in HTML
#
# Version:      1.0.0
# Author:       Bartolomeo Bogliolo mail@meo.bogliolo.name
# Date:         1-NOV-2017
# License:      GPL
#
# Note:
# Init:         1-NOV-2017 meo@bogliolo.name
#               Initial version based on Motion V1 statistics

USR=root
# Careful with security, Eugene!!
PSS=
HST=127.0.0.1
HSTN=`hostname`
PRT=3306

echo '<!DOCTYPE html><html lang="en"><head><meta charset="UTF-8" /><link rel="stylesheet" href="ux3.css" /> <title>' $HSTN : $PRT \
   ' - XCally Motion (2.0) Statistics</title></head><body>' > $HSTN.Motion2.htm

mysql --user=$USR --password=$PSS --host=$HST --port=$PRT \
   --force --skip-column-names >> $HSTN.Motion2.htm <<EOF

use information_schema;

select '<h1>XCally Motion Report</h1>';

select '<P><A NAME="top"></A>' ;
select '<p>Table of contents:' ;
select '<table><tr><td><ul>' ;
select '<li><A HREF="#status">MySQL Summary</A></li>' ;
select '<li><A HREF="#xcally">XCally</A></li>' ;
select '  <ul><li><A HREF="#cdr">CDR Today</A></li>' ;
select '      <li><A HREF="#cdr">CDR History</A></li>' ;
select '      <li><A HREF="#lic">License</A></li>' ;
select '      <li><A HREF="#team">Teams</A></li>' ;
select '      <li><A HREF="#usr">Users</A></li></ul>' ;
select '<li><A HREF="#prc">MySQL Detailed Status</A></li>' ;
select '</ul></table><p><hr>' ;
 
select '<P>Statistics generated on: ', now();
select ' by: ', user(), 'as: ',current_user();
 
select 'using: <I><b>motion2.sh</b> v.1.0.0';
select '<br>Software by ';
select '<A HREF="http://www.xenialab.it/meo/web/index5.htm#dwn">Meo</A></I><p><HR>';

select '<P><A NAME="status"></A>';
select '<P><table border="2"><tr><td><b>MySQL Summary</b></td></tr>';
select '<tr><td><b>Item</b>', '<td><b>Value</b>';

select '<tr><td>Version :', '<td>', version()
union
select '<tr><td>Created :', '<td>', min(create_time)
from tables
union
select '<tr><td>Started :', '<td>', date_format(date_sub(now(), INTERVAL variable_value second),'%Y-%m-%d %T')
from global_status
where variable_name='UPTIME'
union
select '<tr><td>DB Size (MB) :',
	'<td align=right>',
        format(sum(data_length+index_length)/(1024*1024),0)
from tables
union
select '<tr><td>Caches Size (MB) :',
	'<td align="right">',
	format(sum(variable_value+0)/(1024*1024),0)
from global_variables
where lower(variable_name) like '%buffer_size' or lower(variable_name) like '%buffer_pool_size'
union
select '<tr><td>Logging Bin. :', '<td>', variable_value
from global_status
where variable_name='LOG_BIN'
union
select '<tr><td>Defined Users (Schema) :',
 '<td align="right">', format(count(*),0)
from mysql.user
union
select '(', count(*), ')'
from schemata
where schema_name not in ('information_schema')
union
select '<tr><td>Defined Tables :',
	'<td align=right>', format(count(*),0)
from tables
union
select '<tr><td>Sessions (Active):', '<td align="right">', format(count(*),0)
  from processlist
 union
select '(', format(count(*),0), ')'
  from processlist
 where command <> 'Sleep'
union
select '<tr><td>Questions (#/sec.) :',
 '<td align=right>', format(g1.variable_value/g2.variable_value,5)
  from global_status g1,global_status g2
 where g1.variable_name='QUESTIONS'
   and g2.variable_name='UPTIME'
union
select '<tr><td>BinLog Writes Day (MB) :',
 '<td align=right>', format((g1.variable_value*60*60*24)/(g2.variable_value*1024*1024),0)
  from global_status g1,global_status g2
 where g1.variable_name='INNODB_OS_LOG_WRITTEN'
   and g2.variable_name='UPTIME'
union
select '<tr><td>Hostname:Port :', '<td>', variable_value
  from information_schema.global_variables
 where variable_name ='hostname'
union
select ':', '', variable_value
  from information_schema.global_variables
 where variable_name ='port';
select '</table><p><hr>' ;

select '<P><A NAME="xcally"></A>' ;
select '<P><A NAME="cdr"></A>' ;
select '<P><table border="2"><tr><td><b>CDR (by hour current day)</b></td></tr>' ;
select '<tr><td><b>Time</b><td><b>Inbound</b><td><b>Outbound</b>';
select '    <td><b>Internal</b><td><b>TOTAL</b>';
select '<tr><td>',hour(starttime), 
       '<td>',sum(if(type='inbound',1,0)) inbound, '<td>',sum(if(type='outbound',1,0)) outbound,
       '<td>',sum(if(type='internal',1,0)) internal,
       '<td>',count(*) total      
  from motion2.report_call
 group by day(starttime), hour(starttime)
 order by day(starttime), hour(starttime);
select '</table><p>' ;

select '<P><A NAME="hcdr"></A>' ;
select '<P><table border="2"><tr><td><b>CDR (by day last month)</b></td></tr>' ;
select '<tr><td><b>Time</b><td><b>Inbound</b><td><b>Outbound</b>';
select '    <td><b>Internal</b><td><b>TOTAL</b>';
select '<tr><td>',date_format(starttime,'%Y-%m-%d'), 
       '<td>',sum(if(type='inbound',1,0)) inbound, '<td>',sum(if(type='outbound',1,0)) outbound,
       '<td>',sum(if(type='internal',1,0)) internal,
       '<td>',count(*) total      
  from motion2.report_call_history
 where starttime > DATE_SUB(now(), INTERVAL 35 DAY)
 group by date_format(starttime,'%Y-%m-%d')
 order by date_format(starttime,'%Y-%m-%d');
select '</table><p><hr>' ;

select '<P><A NAME="lic"></A>' ;
select '<P><table border="2"><tr><td><b>License</b></td></tr>' ;
select '<tr><td><b>License</b><td><b>Created</b><td><b>Updated</b>';
select '<tr><td>',data1, '<td>',createdAt, '<td>',updatedAt
  from motion2.license;
select '</table><p><hr>' ;

select '<P><A NAME="team"></A>' ;
select '<P><table border="2"><tr><td><b>Teams</b></td></tr>' ;
select '<tr><td><b>Team</b><td><b>Is default</b>';
select '<tr><td>',name, '<td>',defaultEntry
  from motion2.teams;
select '</table><p>' ;

select '<P><A NAME="usr"></A>' ;
select '<P><table border="2"><tr><td><b>User Status Rollup</b></td></tr>' ;
select '<tr><td><b>Count</b><td><b>Role</b><td><b>Status</b>';
select '<tr><td>',count(*), '<td>',role, '<td>',status
  from motion2.users
 group by role, status with rollup;
select '</table><p><hr>' ;

select '<P><A NAME="prc"></A>' ;
select '<P><table border="2"><tr><td><b>Per-User Processes</b></td></tr>' ;
select '<tr><td><b>User</b><td><b>Database</b><td><b>Count</b>';
select '<tr><td>', user,
	'<td>', db,
	'<td>', count(*)
from processlist
group by user, db
order by 6 desc;
select '<tr><td>TOTAL (', count(distinct user),
	' distinct users)<td>',
	'<td>', count(*)
from processlist;
select '</table><p>';

select '<P><A NAME="tbs"></A>' ;
select '<P><table border="2"><tr><td><b>Space Usage</b></td></tr>' ;
select '<tr><td><b>Database',
 '<td><b>Row#</b>',
 '<td><b>Data size</b>',
 '<td><b>Index size</b>',
 '<td><b>Total size</b>',
 '<td><b></b>',
 '<td><b>MyISAM</b>',
 '<td><b>InnoDB</b>',
 '<td><b>Created</b>';
select '<tr><td>', table_schema,
	'<td align=right>', format(sum(table_rows),0),
	'<td align=right>', format(sum(data_length),0),
	'<td align=right>', format(sum(index_length),0),
	'<td align=right>', format(sum(data_length+index_length),0),
	'<td>',
	'<td align=right>', format(sum((data_length+index_length)*
		if(engine='MyISAM',1,0)),0),
	'<td align=right>', format(sum((data_length+index_length)*
		if(engine='InnoDB',1,0)),0),
	'<td>', date_format(min(create_time),'%Y-%m-%d')
from tables
group by table_schema with rollup;
select '</table><p>' ;

select '<P><table border="2"><tr><td><b>Tuning Parameters (most used ones)</b></td></tr>';
select '<tr><td><b>Parameter</b>',
 '<td><b>Value</b><td><b>Type</b>' ;
select '<tr><td>', variable_name, '<td align=right>', variable_value, '<td>Flag'
from global_variables
where lower(variable_name) in (
'log_bin',
'query_cache_type',
'slow_query_log',
'foo')
union
select '<tr><td>', variable_name, '<td align=right>', format(variable_value,0), '<td>Cache'
from global_variables
where lower(variable_name) in (
'innodb_buffer_pool_size',
'query_cache_size',
'innodb_additional_mem_pool_size',
'innodb_log_file_size',
'innodb_log_buffer_size',
'key_buffer_size',
'table_open_cache',
'tmp_table_size',
'max_heap_table_size',
'foo')
union
select '<tr><td>', variable_name, '<td align=right>', format(variable_value,0), '<td>Tuning and timeout'
from global_variables
where lower(variable_name) in (
'innodb_flush_log_at_trx_commit',
'innodb_flush_log_at_timeout',
'innodb_log_files_in_group',
'innodb_lock_wait_timeout',
'innodb_thread_concurrency',
'skip-external-locking',
'wait_timeout',
'long_query_time',
'sync_binlog',
'foo')
union
select '<tr><td>', variable_name, '<td align=right>', format(variable_value,0), '<td>Client Cache'
from global_variables
where lower(variable_name) in (
'binlog_cache_size',
'binlog_stmt_cache_size',
'max_connections',
'read_buffer_size',
'read_rnd_buffer_size',
'sort_buffer_size',
'join_buffer_size',
'thread_stack',
'foo')
order by variable_name;
select '</table><p><hr>' ;

select '<hr><P>Statistics generated on: ', now();
select '<p>For more info on motion report contact' ;
select '<A HREF="mailto:mail@meo.bogliolo.name">Meo Bogliolo</A>.<p></body></html>' ;

EOF
