#! /bin/sh
# ux2p-XXX-OLVM
# by mail@meo.bogliolo.name (c)
#
# Oracle Linux Virtualization Manager Engine/Node Plugin
# Reports Oracle Linux Virtualization Manager configuration details (KVM)
#
# Usage:
# called by ux2html.sh 
#
# Notes:
# OLVM version 4.2 or newer
# History:
#  14 Feb 2020 1.0.0		meo	First version
#  28 Jul 2022 1.0.1            meo	First revision
#  28 Jun 2023 1.0.2b           meo	Few fixings (a) certificates (b) small bug fixed

PL_VERSION=1.0.2b
PL_DESCR="OLVM Engine / KVM Host"

echo '<P><a id="olvm"></A><H2>' $PL_DESCR '</h2>' 
echo 'Plug-in version:' $PL_VERSION

## Plug-in code
#OLVM_PASS=
OERRLEN=20
	
## Report
echo '<h3>Configuration</h3><pre>'
echo
find /etc/ovirt-engine-setup.conf.d -name '*.conf' -exec echo "<b>" {} "</b>" \; -exec cat {} \; -exec echo " " \; | grep -v ^# | grep -v ^$   2>/dev/null
find /etc/ovirt-engine -name '*.conf' -exec echo "<b>" {} "</b>" \; -exec cat {} \; -exec echo "  " \; | grep -v ^# | grep -v ^$   2>/dev/null

find /etc/vdsm -name '*.conf' -exec echo "<b>" {} "</b>" \; -exec cat {} \; -exec echo "  " \; | grep -v ^# | grep -v ^$   2>/dev/null

echo '<b>Certificates</b>'
for i in /etc/pki/ovirt-engine/apache-ca.pem /etc/pki/ovirt-engine/certs/engine.cer /etc/pki/vdsm/certs/vdsmcert.pem
do
  echo $i
  ls -l $i
  openssl x509 -in $i  -noout -text
  echo
done
echo '</pre>'

echo '<h3>Status</h3><pre>'
# journalctl -u ovirt-engine
echo '<b>Engine</b>'
for i in ovirt-engine ovirt-imageio ovirt-engine-dwhd ovirt-fence-kdump-listener ovirt-provider-ovn ovirt-vmconsole-proxy-sshd ovirt-websocket-proxy
do
  systemctl status -l $i
  echo
done

echo '<b>Host</b>'
for i in 'ovirt-imageio-daemon ovirt-vmconsole-host-sshd vdsmd'
do
  systemctl status -l $i
  echo
done

echo '  <b>Processes</b>' 
eval $PS | grep qemu- | cut -c1-120
echo

# hosted-engine --vm-status           2>/dev/null             # hosted-engine non è incluso nella versione per OL7.9
# hosted-engine --check-liveliness    2>/dev/null  
echo '</pre>'


echo '<h3>Logs</h3><pre>'
echo '<b>Engine</b>'
for i in 'ovirt-engine ovirt-imageio ovirt-engine-backup ovirt-engine-dwh ovirt-imageio-proxy'
do
  find /var/log/$i -name '*.log' -exec echo "<b>" {} "</b>" \; -exec tail -$OERRLEN {} \; -exec echo " " \; 2>/dev/null
  echo
done

echo '<b>Host</b>'
for i in 'vdsm libvirt ovirt-imageio-daemon ovirt-hosted-engine-ha ovirt-hosted-engine-setup'
do
  find /var/log/$i -name '*.log' -exec echo "<b>" {} "</b>" \; -exec tail -$OERRLEN {} \; -exec echo " " \; 2>/dev/null
  echo
done
echo '</pre>'

echo '<h3>Packages</h3><pre>'
rpm -qa | grep ovirt
echo '</pre>'

echo '<h3>Engine Details (DB)</h3><pre>'
# The PG database contains a lot of useful informations...
# https://support.oracle.com/epmos/faces/DocumentDisplay?_afrLoop=189180115843572&parent=EXTERNAL_SEARCH&sourceId=HOWTO&id=2690565.1&_afrWindowMode=0&_adf.ctrl-state=zliqjtst_4

/usr/share/ovirt-engine/dbscripts/engine-psql.sh -c "select vds_id, vds_name, vds_unique_id, port,cluster_id , _create_date from vds_static order by vds_id"
/usr/share/ovirt-engine/dbscripts/engine-psql.sh -c "select vds_id, status, cpu_cores, vm_active, vm_count, host_os, pretty_name, hw_family from vds_dynamic order by vds_id"
/usr/share/ovirt-engine/dbscripts/engine-psql.sh -c "select vds_static.vds_name,vds_dynamic.status, case vds_dynamic.status when 0 then 'Unassigned' when 1 then 'Down' when 2 then 'Maintenance' when 3 then 'Up' when 4 then 'NonResponsive' when 5 then 'Error' else 'Other' end Description, vds_dynamic.non_operational_reason from vds_static,vds_dynamic where vds_static.vds_id=vds_dynamic.vds_id  "

/usr/share/ovirt-engine/dbscripts/engine-psql.sh -c "select user_id, name, username from users"
/usr/share/ovirt-engine/dbscripts/engine-psql.sh -c "select id,name,description,storage_pool_id,vdsm_name from network"

/usr/share/ovirt-engine/dbscripts/engine-psql.sh -c "select vm_guid, vm_name, num_of_sockets, mem_size_mb, description, template_status, original_template_name, _create_date from vm_static order by template_status desc, vm_guid"
/usr/share/ovirt-engine/dbscripts/engine-psql.sh -c "select vm_guid, vm_host, runtime_name, status, vm_ip, last_start_time, guest_os, run_on_vds from vm_dynamic order by vm_guid"

/usr/share/ovirt-engine/dbscripts/engine-psql.sh -c "select option_name, option_value, default_value from vdc_options order by option_name"

# select setting::bigint from pg_settings where name='max_connections';
# select pg_size_pretty(shared_buffers::bigint*block_size + max_connections*work_mem*1024 + autovacuum_max_workers*(case when autovacuum_work_mem=-1 then maintenance_work_mem else autovacuum_work_mem end)*1024) as estimated_max_ram_usage from ( select (select setting::bigint from pg_settings where name='block_size') as block_size, (select setting::bigint from pg_settings where name='shared_buffers') as shared_buffers, (select setting::bigint from pg_settings where name='max_connections') as max_connections,(select setting::bigint from pg_settings where name='work_mem') as work_mem, (select setting::bigint from pg_settings where name='autovacuum_max_workers') as autovacuum_max_workers,(select setting::bigint from pg_settings where name='autovacuum_work_mem') as autovacuum_work_mem,(select setting::bigint from pg_settings where name='maintenance_work_mem') as maintenance_work_mem)as _ ;
echo '</pre>'


which virsh > /dev/null 2> /dev/null
RES=$?
if [ $RES -eq 0 ]
then
  echo '<h3>Host Details (virsh)</h3><pre>'
  LC_CTYPE="en_US.UTF-8"
  virsh --readonly version
  virsh --readonly hostname
  virsh --readonly nodeinfo
  virsh --readonly list --all
  virsh --readonly nodecpustats
  virsh --readonly nodememstats

  for i in `virsh --readonly list --all | tail -n +3 | awk '{print $2}' `
  do
    echo $i
    virsh --readonly dominfo $i
    virsh --readonly vcpuinfo $i --pretty
  done

  echo '</pre>'
  echo '<xmp>'
  virsh --readonly sysinfo
  echo
  virsh --readonly capabilities
  echo '</xmp>'
fi

echo '</pre>'

